init python:
    config.per_frame_screens.append('pink_otm_map')

"""Used to cache non-static elements, keeping them as part of your save game. Key is the path to the map, value is a
subdictionary of non-static objects. This subdictionary has the reference name of the object as the key, and the actual
object (usually a pink.otm.CollectionObject) as the value. Initialized here so it resets on a new game.
"""
default pink_otm_variable_cache = {}
default pink_otm_event_trigger = None
default pink_otm_current_event_name = None
default pink_otm_current_event_type = None
default pink_otm_event_interaction_reaction = None
default pink_otm_pc_init_dict = {  # Used to initiate the player character.
    "properties": [
        {"name": "sprite_collection_path", "type": "string", "value": pink_otm_default_sprite},
        {"name": "repeat_commands", "type": "bool", "value": False},
        {"name": "movement_speed", "type": "float", "value": pink_otm_default_pc_walk_speed},
        {"name": "quick_movement_animation", "type": "str", "value": "run"},
        {"name": "default_movement_animation", "type": "str", "value": "walk"},
        {"name": "ref_name", "type": "str", "value": "pink_otm_current_pc"}]}
default pink_otm_camera_drone_init_dict = {
    "properties": [
        {"name": "sprite_collection_path", "type": "string",
         "value": "pink_engine/sprite_collections/empty_sprite_collection.json"},
        {"name": "can_always_move", "type": "bool", "value": True},
        {"name": "repeat_commands", "type": "bool", "value": False},
        {"name": "stand_animation", "type": "string", "value": "exist"},
        {"name": "move_animation", "type": "string", "value": "exist"},
        {"name": "ref_name", "type": "str", "value": "pink_otm_camera_drone"}]}
default pink_otm_current_walk_speed = pink_otm_default_pc_walk_speed
default pink_otm_current_run_speed = pink_otm_default_pc_run_speed

default pink_otm_current_map = None
default pink_otm_current_camera = None

# Setting caches
default pink_otm_rollback_restore = True
default pink_otm_window_hide_cache = None
default pink_otm_autosave_on_choice_cache = None

# Followers
default pink_otm_followers = []

default pink_otm_zoom_controller = pink.otm.SmoothZoomController()
default pink_otm_global_parallel_processes = [
    pink_otm_zoom_controller
]

# Keymap
default pink_otm_event_type_functions = {
    pygame.KEYDOWN: pink.otm.OrthogonalTiledMap.event_key_down,
    pygame.KEYUP: pink.otm.OrthogonalTiledMap.event_key_up,
    pygame.JOYBUTTONDOWN: pink.otm.OrthogonalTiledMap.event_joy_button_down,
    pygame.JOYBUTTONUP: pink.otm.OrthogonalTiledMap.event_joy_button_up,
    pygame.JOYAXISMOTION: pink.otm.OrthogonalTiledMap.event_joy_axis_motion,
    pygame.JOYBALLMOTION: pink.otm.OrthogonalTiledMap.event_joy_ball_motion,
    pygame.JOYHATMOTION: pink.otm.OrthogonalTiledMap.event_joy_hat_motion,
    pygame.MOUSEBUTTONDOWN: pink.otm.OrthogonalTiledMap.event_mouse_button_down,
    pygame.MOUSEBUTTONUP: pink.otm.OrthogonalTiledMap.event_mouse_button_up}

default pink_otm_joy_button_down_functions = {
    0: pink.otm.OrthogonalTiledMap.player_interaction,
    1: pink.otm.OrthogonalTiledMap.run_key_down}

default pink_otm_joy_button_up_functions = {
    1: pink.otm.OrthogonalTiledMap.run_key_up}

default pink_otm_mouse_button_down_functions = {
    1: pink.otm.OrthogonalTiledMap.player_interaction}

default pink_otm_mouse_button_up_functions = {}

default pink_otm_key_down_functions = {
    pygame.K_SPACE: pink.otm.OrthogonalTiledMap.player_interaction,
    pygame.K_RETURN: pink.otm.OrthogonalTiledMap.player_interaction,
    pygame.K_RSHIFT: pink.otm.OrthogonalTiledMap.run_key_down,
    pygame.K_LSHIFT: pink.otm.OrthogonalTiledMap.run_key_down,
    pygame.K_o: pink.otm.OrthogonalTiledMap.console_key_down}

default pink_otm_key_up_functions = {
    pygame.K_RSHIFT: pink.otm.OrthogonalTiledMap.run_key_up,
    pygame.K_LSHIFT: pink.otm.OrthogonalTiledMap.run_key_up}

default pink_otm_key_held_functions = {
    pygame.K_RSHIFT: pink.otm.OrthogonalTiledMap.start_running,
    pygame.K_LSHIFT: pink.otm.OrthogonalTiledMap.start_running}

default pink_otm_movement_keys = {
    pygame.K_DOWN: 'down', pygame.K_s: 'down',
    pygame.K_UP: 'up', pygame.K_w: 'up',
    pygame.K_LEFT: 'left', pygame.K_a: 'left',
    pygame.K_RIGHT: 'right', pygame.K_d: 'right'}
default pink_otm_alt_turning_enabled = True

default pink_otm_force_run = False
default pink_otm_forbid_run = False

transform pink_otm_size_offset(screen_width, screen_height, image_size_x, image_size_y, xzoom, yzoom, screen_x_offset, screen_y_offset):
    # Centers small screens
    xoffset max(int((screen_width - (image_size_x * xzoom)) / 2), screen_x_offset)
    yoffset max(int((screen_height - (image_size_y * yzoom)) / 2), screen_y_offset)

transform pink_otm_map_zoom(xzoom, yzoom):
    xzoom xzoom
    yzoom yzoom


label pink_otm_call_map:
    scene black

    python:
        # Blocks rollback while on a pink engine map.
        if config.rollback_enabled:
            pink_otm_rollback_restore = True
        else:
            pink_otm_rollback_restore = False

        # Restores config.window_hide_transition to cached value if set.
        if pink_otm_window_hide_cache is not None:
            config.window_hide_transition = pink_otm_window_hide_cache
            pink_otm_window_hide_cache = None

        # Sets config.autosave_on_choice to pink_otm value, caches the original (original is restored on leave_otm)
        if pink_otm_autosave_on_choice_cache is None:
            pink_otm_autosave_on_choice_cache = config.autosave_on_choice
            config.autosave_on_choice = pink_otm_autosave_on_choice

        pink_otm_current_camera = pink_otm_current_map.camera
        pink_otm_current_camera.camera_map_refresh()

        config.rollback_enabled = False

    call screen pink_otm_map

screen pink_otm_map:
    $ map_camera = pink_otm_current_map.camera
    $ displayable = pink_otm_current_map.get_displayable()

    viewport:
        xadjustment map_camera.x
        yadjustment map_camera.y

        at pink_otm_size_offset(
            config.screen_width, config.screen_height, pink_otm_current_map.image_size.x,
            pink_otm_current_map.image_size.y, map_camera.xzoom, map_camera.yzoom, map_camera.viewport_x_offset,
            map_camera.viewport_y_offset)
        id "map_viewport"
        area (0, 0, map_camera.viewport_width, map_camera.viewport_height)

        fixed:
            xsize pink_otm_current_map.image_size.x
            ysize pink_otm_current_map.image_size.y
            at pink_otm_map_zoom(map_camera.xzoom, map_camera.yzoom)
            add displayable

    add pink_otm_current_map